@@ -1,6 +1,6 @@
 /*****************************************************************************
    driverevents.c - Events from the driver processing
-   Copyright (C) 2000 by Jaroslav Kysela <perex@suse.cz>
+   Copyright (C) 2000 by Jaroslav Kysela <perex@perex.cz>
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 ******************************************************************************/
 
 #include "envy24control.h"
@@ -48,6 +48,8 @@ void control_input_callback(gpointer data, gint source, GdkInputCondition condit
 			spdif_output_update();
 		else if (!strcmp(name, "Multi Track Internal Clock"))
 			master_clock_update();
+		else if (!strcmp(name, "Multi Track Internal Clock Default"))
+			master_clock_update();
 		else if (!strcmp(name, "Multi Track Rate Locking"))
 			rate_locking_update();
 		else if (!strcmp(name, "Multi Track Rate Reset"))
