@@ -1,6 +1,6 @@
 /*****************************************************************************
    hardware.c - Hardware Settings
-   Copyright (C) 2000 by Jaroslav Kysela <perex@suse.cz>
+   Copyright (C) 2000 by Jaroslav Kysela <perex@perex.cz>
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -14,12 +14,13 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 ******************************************************************************/
 
 #include "envy24control.h"
 
 static snd_ctl_elem_value_t *internal_clock;
+static snd_ctl_elem_value_t *internal_clock_default;
 static snd_ctl_elem_value_t *word_clock_sync;
 static snd_ctl_elem_value_t *rate_locking;
 static snd_ctl_elem_value_t *rate_reset;
@@ -31,6 +32,8 @@ static snd_ctl_elem_value_t *breakbox_led;
 static snd_ctl_elem_value_t *spdif_on_off;
 static snd_ctl_elem_value_t *phono_input;
 
+static inline int is_update_needed(void);
+
 #define toggle_set(widget, state) \
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), state);
 
@@ -41,10 +44,12 @@ static int is_active(GtkWidget *widget)
 
 void master_clock_update(void)
 {
-	int err, rate;
+	int err, rate, need_default_update;
 	
 	if ((err = snd_ctl_elem_read(ctl, internal_clock)) < 0)
 		g_print("Unable to read Internal Clock state: %s\n", snd_strerror(err));
+	if ((err = snd_ctl_elem_read(ctl, internal_clock_default)) < 0)
+		g_print("Unable to read Internal Clock Default state: %s\n", snd_strerror(err));
 	if (card_eeprom.subvendor == ICE1712_SUBDEVICE_DELTA1010 ||
 	    card_eeprom.subvendor == ICE1712_SUBDEVICE_DELTA1010LT) {
 		if ((err = snd_ctl_elem_read(ctl, word_clock_sync)) < 0)
@@ -58,7 +63,12 @@ void master_clock_update(void)
 		}
 	} else {
 //		toggle_set(hw_master_clock_xtal_radio, TRUE);
-		rate = snd_ctl_elem_value_get_enumerated(internal_clock, 0);
+		need_default_update = !is_update_needed() ? 1 : 0;
+		if (need_default_update) {
+			rate = snd_ctl_elem_value_get_enumerated(internal_clock_default, 0);
+		} else {
+			rate = snd_ctl_elem_value_get_enumerated(internal_clock, 0);
+		}
 		switch (rate) {
 		case 5: toggle_set(hw_master_clock_xtal_22050, TRUE); break;
 		case 7: toggle_set(hw_master_clock_xtal_32000, TRUE); break;
@@ -118,6 +128,7 @@ void internal_clock_toggled(GtkWidget *togglebutton, gpointer data)
 	} else if (!strcmp(what, "SPDIF")) {
 		internal_clock_set(13);
 	} else if (!strcmp(what, "WordClock")) {
+		internal_clock_set(13); // 13 should be considered '!internal'
 		master_clock_word_select(1);
 	} else {
 		g_print("internal_clock_toggled: %s ???\n", what);
@@ -152,15 +163,15 @@ gint master_clock_status_timeout_callback(gpointer data)
 	snd_ctl_elem_value_t *sw;
 	int err;
 	
-	if (card_eeprom.subvendor != ICE1712_SUBDEVICE_DELTA1010)
+	if (card_eeprom.subvendor != ICE1712_SUBDEVICE_DELTA1010 && card_eeprom.subvendor != ICE1712_SUBDEVICE_DELTA1010LT)
 		return FALSE;
 	snd_ctl_elem_value_alloca(&sw);
-	snd_ctl_elem_value_set_interface(sw, SND_CTL_ELEM_IFACE_PCM);
+	snd_ctl_elem_value_set_interface(sw, SND_CTL_ELEM_IFACE_MIXER);
 	snd_ctl_elem_value_set_name(sw, "Word Clock Status");
 	if ((err = snd_ctl_elem_read(ctl, sw)) < 0)
 		g_print("Unable to determine word clock status: %s\n", snd_strerror(err));
 	gtk_label_set_text(GTK_LABEL(hw_master_clock_status_label),
-			   snd_ctl_elem_value_get_boolean(sw, 0) ? "Locked" : "No signal");
+			   snd_ctl_elem_value_get_boolean(sw, 0) ? "No signal" : "Locked");
 	return TRUE;
 }
 
@@ -171,6 +182,8 @@ gint internal_clock_status_timeout_callback(gpointer data)
 	
 	if ((err = snd_ctl_elem_read(ctl, internal_clock)) < 0)
 		g_print("Unable to read Internal Clock state: %s\n", snd_strerror(err));
+	if ((err = snd_ctl_elem_read(ctl, internal_clock_default)) < 0)
+		g_print("Unable to read Internal Clock Default state: %s\n", snd_strerror(err));
 	if (card_eeprom.subvendor == ICE1712_SUBDEVICE_DELTA1010 ||
 	    card_eeprom.subvendor == ICE1712_SUBDEVICE_DELTA1010LT) {
 		if ((err = snd_ctl_elem_read(ctl, word_clock_sync)) < 0)
@@ -218,11 +231,45 @@ gint internal_clock_status_timeout_callback(gpointer data)
 			    g_print("Error in rate: %d\n", rate);
 			    break;
 		}
+		if (!need_update) {	//default clock need update
+			rate = snd_ctl_elem_value_get_enumerated(internal_clock_default, 0);
+			switch (rate) {
+			case 5: toggle_set(hw_master_clock_xtal_22050, TRUE); break;
+			case 7: toggle_set(hw_master_clock_xtal_32000, TRUE); break;
+			case 8: toggle_set(hw_master_clock_xtal_44100, TRUE); break;
+			case 9: toggle_set(hw_master_clock_xtal_48000, TRUE); break;
+			case 11: toggle_set(hw_master_clock_xtal_88200, TRUE); break;
+			case 12: toggle_set(hw_master_clock_xtal_96000, TRUE); break;
+			default:
+				g_print("Error in rate: %d\n", rate);
+				break;
+			}
+		}
 	}
 	gtk_label_set_text(GTK_LABEL(hw_master_clock_actual_rate_label), label);
 	return TRUE;
 }
 
+gint rate_locking_status_timeout_callback(gpointer data)
+{
+    int state;
+
+    if (is_active(hw_rate_locking_check) != (state = is_rate_locked())) {
+	toggle_set(hw_rate_locking_check, state ? TRUE : FALSE);
+    }
+    return TRUE;
+}
+
+gint rate_reset_status_timeout_callback(gpointer data)
+{
+    int state;
+
+    if (is_active(hw_rate_reset_check) != (state = is_rate_reset())) {
+	toggle_set(hw_rate_reset_check, state ? TRUE : FALSE);
+    }
+    return TRUE;
+}
+
 void rate_locking_update(void)
 {
 	int err;
@@ -559,17 +606,30 @@ void spdif_output_toggled(GtkWidget *togglebutton, gpointer data)
 void spdif_input_update(void)
 {
 	int err;
-	
-	if ((card_eeprom.subvendor != ICE1712_SUBDEVICE_DELTADIO2496) && (card_eeprom.subvendor != ICE1712_SUBDEVICE_DMX6FIRE))
+	int digoptical = FALSE;
+	int diginternal = FALSE;
+
+	if ((card_eeprom.subvendor != ICE1712_SUBDEVICE_DELTADIO2496) &&
+	    ! card_is_dmx6fire)
 		return;
 	if ((err = snd_ctl_elem_read(ctl, spdif_input)) < 0)
 		g_print("Unable to read S/PDIF input switch: %s\n", snd_strerror(err));
-	if (snd_ctl_elem_value_get_boolean(spdif_input, 0)) {
+	if (snd_ctl_elem_value_get_boolean(spdif_input, 0))
+		digoptical = TRUE;
+	if (card_is_dmx6fire) {
+        	if ((err = snd_ctl_elem_read(ctl, spdif_on_off)) < 0)
+			g_print("Unable to read S/PDIF on/off switch: %s\n", snd_strerror(err));
+	      	if (!(snd_ctl_elem_value_get_boolean(spdif_on_off, 0)))
+			diginternal = TRUE;
+	}
+	if (digoptical) {
 		toggle_set(hw_spdif_input_optical_radio, TRUE);
 	} else {
 		toggle_set(hw_spdif_input_coaxial_radio, TRUE);
 	}
-}
+	if (diginternal)
+		toggle_set(hw_spdif_switch_off_radio, TRUE);
+ }
 
 void spdif_input_toggled(GtkWidget *togglebutton, gpointer data)
 {
@@ -578,10 +638,18 @@ void spdif_input_toggled(GtkWidget *togglebutton, gpointer data)
 	
 	if (!is_active(togglebutton))
 		return;
-	if (!strcmp(str, "Optical"))
-		snd_ctl_elem_value_set_boolean(spdif_input, 0, 1);
-	else
-		snd_ctl_elem_value_set_boolean(spdif_input, 0, 0);
+	if (!strcmp(str, "Off"))
+               	snd_ctl_elem_value_set_boolean(spdif_on_off, 0, 0);
+	else {
+		snd_ctl_elem_value_set_boolean(spdif_on_off, 0, 1);
+		if (!strcmp(str, "Optical"))
+			snd_ctl_elem_value_set_boolean(spdif_input, 0, 1);
+		else
+			if (!strcmp(str, "Coaxial"))
+				snd_ctl_elem_value_set_boolean(spdif_input, 0, 0);
+	}
+	if ((err = snd_ctl_elem_write(ctl, spdif_on_off)) < 0)
+               g_print("Unable to write S/PDIF on/off switch: %s\n", snd_strerror(err));
 	if ((err = snd_ctl_elem_write(ctl, spdif_input)) < 0)
 		g_print("Unable to write S/PDIF input switch: %s\n", snd_strerror(err));
 }
@@ -590,7 +658,7 @@ void analog_input_select_update(void)
 {
 	int err, input_interface;
 
-	if (card_eeprom.subvendor != ICE1712_SUBDEVICE_DMX6FIRE)
+	if (! card_is_dmx6fire)
 		return;
 	if ((err = snd_ctl_elem_read(ctl, analog_input_select)) < 0)
 		g_print("Unable to read analog input switch: %s\n", snd_strerror(err));
@@ -618,88 +686,34 @@ void analog_input_select_set(int value)
 void analog_input_select_toggled(GtkWidget *togglebutton, gpointer data)
 {
 	char *what = (char *) data;
+       int err;
 
         if (!is_active(togglebutton))
                 return;
         if (!strcmp(what, "Internal")) {
                 analog_input_select_set(0);
+               snd_ctl_elem_value_set_boolean(breakbox_led, 0, 0);
         } else if (!strcmp(what, "Front Input")) {
                 analog_input_select_set(1);
+               snd_ctl_elem_value_set_boolean(breakbox_led, 0, 1);
         } else if (!strcmp(what, "Rear Input")) {
                 analog_input_select_set(2);
+               snd_ctl_elem_value_set_boolean(breakbox_led, 0, 0);
         } else if (!strcmp(what, "Wave Table")) {
                 analog_input_select_set(3);
+               snd_ctl_elem_value_set_boolean(breakbox_led, 0, 0);
         } else {
                 g_print("analog_input_select_toggled: %s ???\n", what);
         }
-}
-
-
-void breakbox_led_update(void)
-{
-	int err;
-
-	 if (card_eeprom.subvendor != ICE1712_SUBDEVICE_DMX6FIRE)
-		return;
-	if ((err = snd_ctl_elem_read(ctl, breakbox_led)) < 0)
-		g_print("Unable to read breakbox LED switch: %s\n", snd_strerror(err));
-	if (snd_ctl_elem_value_get_boolean(breakbox_led, 0)) {
-		 toggle_set(hw_breakbox_led_on_radio, TRUE);
-	} else {
-		 toggle_set(hw_breakbox_led_off_radio, TRUE);
-	}
-}
-
-void breakbox_led_toggled(GtkWidget *togglebutton, gpointer data)
-{
-	int err;
-		char *str = (char *)data;
-
-	 if (!is_active(togglebutton))
-		return;
-	if (!strcmp(str, "On"))
-			snd_ctl_elem_value_set_boolean(breakbox_led, 0, 1);
-	else
-			snd_ctl_elem_value_set_boolean(breakbox_led, 0, 0);
-	if ((err = snd_ctl_elem_write(ctl, breakbox_led)) < 0)
-		g_print("Unable to write breakbox LED switch: %s\n", snd_strerror(err));
-}
-
-void spdif_on_off_update(void)
-{
-        int err;
-
-        if (card_eeprom.subvendor != ICE1712_SUBDEVICE_DMX6FIRE)
-                return;
-        if ((err = snd_ctl_elem_read(ctl, spdif_on_off)) < 0)
-                g_print("Unable to read S/PDIF on/off switch: %s\n", snd_strerror(err));
-        if (snd_ctl_elem_value_get_boolean(spdif_on_off, 0)) {
-                toggle_set(hw_spdif_switch_on_radio, TRUE);
-        } else {
-                toggle_set(hw_spdif_switch_off_radio, TRUE);
-        }
-}
-
-void spdif_on_off_toggled(GtkWidget *togglebutton, gpointer data)
-{
-        int err;
-        char *str = (char *) data;
-
-        if (!is_active(togglebutton))
-                return;
-        if (!strcmp(str, "On"))
-                snd_ctl_elem_value_set_boolean(spdif_input, 0, 1);
-        else
-                snd_ctl_elem_value_set_boolean(spdif_input, 0, 0);
-        if ((err = snd_ctl_elem_write(ctl, spdif_input)) < 0)
-                g_print("Unable to write S/PDIF on/off switch: %s\n", snd_strerror(err));
+       if ((err = snd_ctl_elem_write(ctl, breakbox_led)) < 0)
+               g_print("Unable to write breakbox LED switch: %s\n", snd_strerror(err));
 }
 
 void phono_input_update(void)
 {
         int err;
 
-        if (card_eeprom.subvendor != ICE1712_SUBDEVICE_DMX6FIRE)
+        if (! card_is_dmx6fire)
                 return;
         if ((err = snd_ctl_elem_read(ctl, phono_input)) < 0)
                 g_print("Unable to read phono input switch: %s\n", snd_strerror(err));
@@ -728,6 +742,7 @@ void phono_input_toggled(GtkWidget *togglebutton, gpointer data)
 void hardware_init(void)
 {
 	if (snd_ctl_elem_value_malloc(&internal_clock) < 0 ||
+	    snd_ctl_elem_value_malloc(&internal_clock_default) < 0 ||
 	    snd_ctl_elem_value_malloc(&word_clock_sync) < 0 ||
 	    snd_ctl_elem_value_malloc(&rate_locking) < 0 ||
 	    snd_ctl_elem_value_malloc(&rate_reset) < 0 ||
@@ -745,7 +760,10 @@ void hardware_init(void)
 	snd_ctl_elem_value_set_interface(internal_clock, SND_CTL_ELEM_IFACE_MIXER);
 	snd_ctl_elem_value_set_name(internal_clock, "Multi Track Internal Clock");
 
-	snd_ctl_elem_value_set_interface(word_clock_sync, SND_CTL_ELEM_IFACE_PCM);
+	snd_ctl_elem_value_set_interface(internal_clock_default, SND_CTL_ELEM_IFACE_MIXER);
+	snd_ctl_elem_value_set_name(internal_clock_default, "Multi Track Internal Clock Default");
+
+	snd_ctl_elem_value_set_interface(word_clock_sync, SND_CTL_ELEM_IFACE_MIXER);
 	snd_ctl_elem_value_set_name(word_clock_sync, "Word Clock Sync");
 
 	snd_ctl_elem_value_set_interface(rate_locking, SND_CTL_ELEM_IFACE_MIXER);
@@ -757,12 +775,12 @@ void hardware_init(void)
 	snd_ctl_elem_value_set_interface(volume_rate, SND_CTL_ELEM_IFACE_MIXER);
 	snd_ctl_elem_value_set_name(volume_rate, "Multi Track Volume Rate");
 
-	if(card_eeprom.subvendor == ICE1712_SUBDEVICE_DMX6FIRE){
+	if (card_is_dmx6fire) {
 		snd_ctl_elem_value_set_interface(spdif_input, SND_CTL_ELEM_IFACE_MIXER);
 		snd_ctl_elem_value_set_name(spdif_input, "Optical Digital Input Switch");
 	} else {
-	snd_ctl_elem_value_set_interface(spdif_input, SND_CTL_ELEM_IFACE_PCM);
-	snd_ctl_elem_value_set_name(spdif_input, "IEC958 Input Optical");
+		snd_ctl_elem_value_set_interface(spdif_input, SND_CTL_ELEM_IFACE_MIXER);
+		snd_ctl_elem_value_set_name(spdif_input, "IEC958 Input Optical");
 	}
 
 	snd_ctl_elem_value_set_interface(spdif_output, SND_CTL_ELEM_IFACE_PCM);
@@ -791,7 +809,5 @@ void hardware_postinit(void)
 	spdif_input_update();
 	spdif_output_update();
 	analog_input_select_update();
-	breakbox_led_update();
-	spdif_on_off_update();
 	phono_input_update();
 }
