@@ -2,6 +2,9 @@
  *   HDSPMixer
  *    
  *   Copyright (C) 2003 Thomas Charbonnel (thomas@undata.org)
+ *
+ *   Copyright (C) 2011 Adrian Knoth (adi@drcomp.erfurt.thur.de)
+ *                      Fredrik Lingvall (fredrik.lingvall@gmail.com)
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,134 +24,6 @@
 #pragma implementation
 #include "HDSPMixerCard.h"
 
-static char channel_map_df_ss[26] = {
-    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
-    18, 19, 20, 21, 22, 23, 24, 25
-};
-
-static char channel_map_mf_ss[26] = {
-    0, 1, 2, 3, 4, 5, 6, 7,
-    16, 17, 18, 19, 20, 21, 22, 23, 
-    24, 25,
-    -1, -1, -1, -1, -1, -1, -1, -1
-};
-
-static char meter_map_ds[26] = {
-    0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 
-    24, 25,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
-};
-
-static char channel_map_ds[26] = {
-    1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 
-    24, 25,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
-};
-
-static char dest_map_madi_ss[32] = {
-  0,  2,  4,  6,  8, 10, 12, 14,
- 16, 18, 20, 22, 24, 26, 28, 30,
- 32, 34, 36, 38, 40, 42, 44, 46,
- 48, 50, 52, 54, 56, 58, 60, 62
-};
-
-
-static char dest_map_raydat_ss[18] = {
-   4,  6,  8, 10, 
-  12, 14, 16, 18, 
-  20, 22, 24, 26, 
-  28, 30, 32, 34,
-   0,  2
-};
-
-static char dest_map_raydat_ds[10] = {
-   4,  6, 
-   8, 10, 
-  12, 14, 
-  16, 18,
-   0,  2
-};
-
-static char dest_map_raydat_qs[6] = {
-   4,  
-   6,
-   8,
-  10,
-   0,  2
-};
-
-
-
-static char dest_map_aio_ss[8] = {
-   0, // Analogue
-   8, // AES 
-  10, // SPDIF
-  12, 14, 16, 18, // ADAT
-   6  // Phones 
-};
-
-
-static char dest_map_aio_ds[6] = {
-   0, // Analogue 
-   8, // AES
-  10, // SPDIF
-  12, 16, // ADAT
-   6  // Phones
-};
-
-static char dest_map_aio_qs[5] = {
-   0, // Analogue 
-   8, // AES
-  10, // SPDIF
-  12, // ADAT
-   6  // Phone
-};
-
-
-static char dest_map_mf_ss[10] = {
-    0, 2, 4, 6, 16, 18, 20, 22, 24, 26 
-};
-
-static char dest_map_ds[8] = {
-    0, 2, 8, 10, 16, 18, 24, 26 
-};
-
-static char dest_map_df_ss[14] = {
-    0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26 
-};
-
-static char dest_map_h9652_ss[13] = {
-    0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24 
-};
-
-static char dest_map_h9652_ds[7] = {
-    0, 2, 8, 10, 16, 18, 24 
-};
-
-static char dest_map_h9632_ss[8] = {
-    0, 2, 4, 6, 8, 10, 12, 14
-};
-
-static char dest_map_h9632_ds[6] = {
-    0, 2, 8, 10, 12, 14
-};
-
-static char dest_map_h9632_qs[4] = {
-    8, 10, 12, 14
-};
-
-static char channel_map_h9632_ss[16] = {
-    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
-};
-
-static char channel_map_h9632_ds[12] = {
-    0, 1, 2, 3, 8, 9, 10, 11, 12, 13, 14, 15
-};
-
-static char channel_map_h9632_qs[8] = {
-    8, 9, 10, 11, 12, 13, 14, 15
-};
-
 static void alsactl_cb(snd_async_handler_t *handler)
 {
     int err, clock_value;
@@ -288,11 +163,12 @@ int HDSPMixerCard::getSpeed()
     return 0;    
 }
 
-HDSPMixerCard::HDSPMixerCard(int cardtype, int id)
+HDSPMixerCard::HDSPMixerCard(int cardtype, int id, char *shortname)
 {
     type = cardtype;
     card_id = id;
     snprintf(name, 6, "hw:%i", card_id);
+    cardname = shortname;
     h9632_aeb.aebi = 0;
     h9632_aeb.aebo = 0;
     if (type == H9632) {
@@ -310,6 +186,7 @@ HDSPMixerCard::HDSPMixerCard(int cardtype, int id)
     
     /* Set channels and mappings */
     adjustSettings();
+    last_preset = last_dirty = 0;
         
     basew = NULL;
 }
@@ -333,171 +210,227 @@ void HDSPMixerCard::getAeb() {
 
 void HDSPMixerCard::adjustSettings() {
     if (type == Multiface) {
-	switch (speed_mode) {
-	case 0:
-	    channels_input = 18;
-	    channels_playback = 20;
-	    channel_map_input = channel_map_playback = channel_map_mf_ss;
-	    dest_map = dest_map_mf_ss;
-	    meter_map_input = meter_map_playback = channel_map_mf_ss;
-	    break;
-	case 1:
-	    channels_input = 14;
-	    channels_playback = 16;
-	    channel_map_input = channel_map_playback = meter_map_ds;
-	    dest_map = dest_map_ds;
-	    meter_map_input = meter_map_playback = meter_map_ds;
-	    break;
-	case 2:
-	    /* should never happen */
-	    break;
-	}
-    } else if (type == Digiface) {
-	switch (speed_mode) {
-	case 0:
-	    channels_input = channels_playback = 26;
-	    channel_map_input = channel_map_playback = channel_map_df_ss;
-	    dest_map = dest_map_df_ss;
-	    meter_map_input = meter_map_playback = channel_map_df_ss;
-	    break;
-	case 1:
-	    channels_input = channels_playback = 14;
-	    channel_map_input = channel_map_playback = meter_map_ds;
-	    dest_map = dest_map_ds;
-	    meter_map_input = meter_map_playback = meter_map_ds;
-	    break;
-	case 2:
-	    /* should never happen */
-	    break;
-	}
-    } else if (type == H9652) {
-	switch (speed_mode) {
-	case 0:
-	    channels_input = channels_playback = 26;
-	    channel_map_input = channel_map_playback = channel_map_df_ss;
-	    dest_map = dest_map_h9652_ss;
-	    meter_map_input = meter_map_playback = channel_map_df_ss;
-	    break;
-	case 1:
-	    channels_input = channels_playback = 14;
-	    channel_map_input = channel_map_playback = channel_map_ds;
-	    dest_map = dest_map_h9652_ds;
-	    meter_map_input = meter_map_playback = meter_map_ds;
-	    break;
-	case 2:
-	    /* should never happen */
-	    break;
-	}
-    } else if (type == H9632) {
-	switch (speed_mode) {
-	case 0:
-	    channels_input = channels_playback = 12 + ((h9632_aeb.aebi || h9632_aeb.aebo) ? 4 : 0);
-	    channel_map_input = channel_map_playback = channel_map_h9632_ss;
-	    dest_map = dest_map_h9632_ss;
-	    meter_map_input = meter_map_playback = channel_map_h9632_ss;
-	    break;
-	case 1:
-	    channels_input = channels_playback = 8 + ((h9632_aeb.aebi || h9632_aeb.aebo) ? 4 : 0);
-	    channel_map_input = channel_map_playback = channel_map_h9632_ds;
-	    dest_map = dest_map_h9632_ds;
-	    meter_map_input = meter_map_playback = channel_map_h9632_ds;
-	    break;
-	case 2:
-	    channels_input = channels_playback = 4 + ((h9632_aeb.aebi || h9632_aeb.aebo) ? 4 : 0);
-	    channel_map_input = channel_map_playback = channel_map_h9632_qs;
-	    dest_map = dest_map_h9632_qs;
-	    meter_map_input = meter_map_playback = channel_map_h9632_qs;
-	    break;
-	}
-    } else if (HDSPeMADI == type) {
-      playbacks_offset = 64;
+        switch (speed_mode) {
+        case 0:
+            channels_input = 18;
+            channels_playback = 18;
+            channels_output = 20; /* SS 8xAnalog+8xADAT+2xSPDIF+2xHeadphone */
+            channel_map_input = channel_map_playback = channel_map_mf_ss;
+            dest_map = dest_map_mf_ss;
+            meter_map_input = meter_map_playback = channel_map_mf_ss;
+            break;
+        case 1:
+            channels_input = 14;
+            channels_playback = 14;
+            channels_output = 16; /* DS 8xAnalog+4xADAT(SMUX)+2xSPDIF+2xHeadphone */
+            channel_map_input = channel_map_playback = meter_map_ds;
+            dest_map = dest_map_ds;
+            meter_map_input = meter_map_playback = meter_map_ds;
+            break;
+        case 2:
+            /* should never happen */
+            break;
+        }
+    }
+    
+    if (type == Digiface) {
+        switch (speed_mode) {
+        case 0:
+            channels_input = channels_playback = 26;
+            channels_output = 28; /* SS 3x8xADAT+2xSPDIF+2xHeadphone */
+            channel_map_input = channel_map_playback = channel_map_df_ss;
+            dest_map = dest_map_df_ss;
+            meter_map_input = meter_map_playback = channel_map_df_ss;
+            break;
+        case 1:
+            channels_input = channels_playback = 14;
+            channels_output = 16; /* DS 3x4xADAT(SMUX)+2xSPDIF+2xHeadphone */
+            channel_map_input = channel_map_playback = meter_map_ds;
+            dest_map = dest_map_ds;
+            meter_map_input = meter_map_playback = meter_map_ds;
+            break;
+        case 2:
+            /* should never happen */
+            break;
+        }
+    }
 
-      switch (speed_mode) {
-      case 0: // SS
-	channels_input = channels_playback = 64;
-	channel_map_input = channel_map_playback = channel_map_unity_ss;
-	dest_map = dest_map_madi_ss;
-	meter_map_input = meter_map_playback = channel_map_unity_ss;
-	break;
-      case 1: // DS
-	channels_input = channels_playback = 32;
-	channel_map_input = channel_map_playback = channel_map_unity_ss;
-	dest_map = dest_map_madi_ss;
-	meter_map_input = meter_map_playback = channel_map_unity_ss;
-	break;
-      case 2: // QS
-	channels_input = channels_playback = 16;
-	channel_map_input = channel_map_playback = channel_map_unity_ss;
-	dest_map = dest_map_madi_ss;
-	meter_map_input = meter_map_playback = channel_map_unity_ss;
-	break;
-      }
+    if (type == RPM) {
+        /* RPM has no digital audio connectors, hence channel mappings don't
+         * depend on speedmode */
+        channels_input = 5;
+        channels_playback = channels_output = 6; /* 2xMain,2xMon,2xPH */
+        channel_map_input = channel_map_playback = channel_map_rpm;
+        dest_map = dest_map_rpm;
+        meter_map_input = meter_map_playback = channel_map_rpm;
+    }
 
-    } else if (HDSPeAIO == type) {
-      playbacks_offset = 64;
 
-      switch (speed_mode) {
-      case 0: // SS
-	channels_input = 14;
-	channels_playback = 16;
-	channel_map_input = channel_map_aio_in_ss;
-	channel_map_playback = channel_map_aio_out_ss;
-	dest_map = dest_map_aio_ss;
-	meter_map_input = channel_map_aio_in_ss;
-	meter_map_playback = channel_map_aio_out_ss;
-	break;
-      case 1: // DS
-	channels_input = 10;
-	channels_playback = 12;
-	channel_map_input = channel_map_aio_in_ds;
-	channel_map_playback = channel_map_aio_out_ds;
-	dest_map = dest_map_aio_ds;
-	meter_map_input = channel_map_aio_in_ds;
-	meter_map_playback = channel_map_aio_out_ds;
-	break;
-      case 2: // QS
-	channels_input = 8;
-	channels_playback =10;
-	channel_map_input = channel_map_aio_in_qs;
-	channel_map_playback = channel_map_aio_out_qs;
-	dest_map = dest_map_aio_qs;
-	meter_map_input = channel_map_aio_in_qs;
-	meter_map_playback = channel_map_aio_out_qs;
-	break;
-      }
+    if (type == H9652) {
+        switch (speed_mode) {
+        case 0:
+            channels_input = channels_playback = 26;
+            channels_output = 26; /* SS like Digiface, but no Headphones */
+            channel_map_input = channel_map_playback = channel_map_df_ss;
+            dest_map = dest_map_h9652_ss;
+            meter_map_input = meter_map_playback = channel_map_df_ss;
+            break;
+        case 1:
+            channels_input = channels_playback = 14;
+            channels_output = 14; /* DS like Digiface, but no Headphones */
+            channel_map_input = channel_map_playback = channel_map_ds;
+            dest_map = dest_map_h9652_ds;
+            meter_map_input = meter_map_playback = meter_map_ds;
+            break;
+        case 2:
+            /* should never happen */
+            break;
+        }
+    }
 
-    } else if (HDSPeRayDAT == type) {
-      playbacks_offset = 64;
+    if (type == H9632) {
+        switch (speed_mode) {
+        case 0:
+            channels_input = channels_playback = 12 + ((h9632_aeb.aebi || h9632_aeb.aebo) ? 4 : 0);
+            channels_output = channels_playback; /* untested, no idea about this card */
+            channel_map_input = channel_map_playback = channel_map_h9632_ss;
+            dest_map = dest_map_h9632_ss;
+            meter_map_input = meter_map_playback = channel_map_h9632_ss;
+            break;
+        case 1:
+            channels_input = channels_playback = 8 + ((h9632_aeb.aebi || h9632_aeb.aebo) ? 4 : 0);
+            channels_output = channels_playback; /* untested, no idea about this card */
+            channel_map_input = channel_map_playback = channel_map_h9632_ds;
+            dest_map = dest_map_h9632_ds;
+            meter_map_input = meter_map_playback = channel_map_h9632_ds;
+            break;
+        case 2:
+            channels_input = channels_playback = 4 + ((h9632_aeb.aebi || h9632_aeb.aebo) ? 4 : 0);
+            channels_output = channels_playback; /* untested, no idea about this card */
+            channel_map_input = channel_map_playback = channel_map_h9632_qs;
+            dest_map = dest_map_h9632_qs;
+            meter_map_input = meter_map_playback = channel_map_h9632_qs;
+            break;
+        }
+    }
 
-      switch (speed_mode) {
-      case 0: // SS
-	channels_input = 36;
-	channels_playback = 36;
-	channel_map_input = channel_map_playback = channel_map_raydat_ss;
-	dest_map = dest_map_raydat_ss;
-	meter_map_input = meter_map_playback = channel_map_raydat_ss;
-	break;
-      case 1: // DS
-	channels_input = 20;
-	channels_playback = 20;
-	channel_map_input = channel_map_playback = channel_map_raydat_ds;
-	dest_map = dest_map_raydat_ds;
-	meter_map_input = meter_map_playback = channel_map_raydat_ds;
-	break;
-      case 2: // QS
-	channels_input = 12;
-	channels_playback =12;
-	channel_map_input = channel_map_playback = channel_map_raydat_qs;
-	dest_map = dest_map_raydat_qs;
-	meter_map_input = meter_map_playback = channel_map_raydat_qs;
-	break;
-      }
+    if (HDSPeMADI == type) {
+        playbacks_offset = 64;
+
+        switch (speed_mode) {
+        case 0: // SS
+            channels_input = channels_playback = 64;
+            channels_output = channels_input; /* SS headphones missing, at least HDSPe MADI has some, MADIface hasn't */
+            channel_map_input = channel_map_playback = channel_map_unity_ss;
+            dest_map = dest_map_unity;
+            meter_map_input = meter_map_playback = channel_map_unity_ss;
+            break;
+        case 1: // DS
+            channels_input = channels_playback = 32;
+            channels_output = channels_input; /* DS headphones missing, at least HDSPe MADI has some, MADIface hasn't */
+            channel_map_input = channel_map_playback = channel_map_unity_ss;
+            dest_map = dest_map_unity;
+            meter_map_input = meter_map_playback = channel_map_unity_ss;
+            break;
+        case 2: // QS
+            channels_input = channels_playback = 16;
+            channels_output = channels_input; /* QS headphones missing, at least HDSPe MADI has some, MADIface hasn't */
+            channel_map_input = channel_map_playback = channel_map_unity_ss;
+            dest_map = dest_map_unity;
+            meter_map_input = meter_map_playback = channel_map_unity_ss;
+            break;
+        }
+
+    }
+
+    if (HDSPeAIO == type) {
+        playbacks_offset = 64;
+
+        switch (speed_mode) {
+        case 0: // SS
+            channels_input = 18;
+            channels_playback = 20;
+            channels_output = 20; /* SS 2xAnalog+2xAES+2xSPDIF+8xADAT+2xHeadphones+4xAEB */
+            channel_map_input = channel_map_aio_in_ss;
+            channel_map_playback = channel_map_aio_out_ss;
+            dest_map = dest_map_aio_ss;
+            meter_map_input = channel_map_aio_in_ss;
+            meter_map_playback = channel_map_aio_out_ss;
+            break;
+        case 1: // DS
+            channels_input = 14;
+            channels_playback = 16;
+            channels_output = 16; /* DS 2xAnalog+2xAES+2xSPDIF+4xADAT(SMUX)+2xHeadphones+4xAEB */
+            channel_map_input = channel_map_aio_in_ds;
+            channel_map_playback = channel_map_aio_out_ds;
+            dest_map = dest_map_aio_ds;
+            meter_map_input = channel_map_aio_in_ds;
+            meter_map_playback = channel_map_aio_out_ds;
+            break;
+        case 2: // QS
+            channels_input = 12;
+            channels_playback = 14;
+            channels_output = 14; /* QS 2xAnalog+2xAES+2xSPDIF+2xADAT(SMUX)+2xHeadphones+4xAEB */
+            channel_map_input = channel_map_aio_in_qs;
+            channel_map_playback = channel_map_aio_out_qs;
+            dest_map = dest_map_aio_qs;
+            meter_map_input = channel_map_aio_in_qs;
+            meter_map_playback = channel_map_aio_out_qs;
+            break;
+        }
+
+    }
+
+    if (HDSP_AES == type) {
+        playbacks_offset = 64; /* not sure about this one? */
+
+        /* 16 channels for all modes */
+        channels_input = 16;
+        channels_playback = 16;
+        channels_output = 16;
+        channel_map_input = channel_map_aes32;
+        channel_map_playback = channel_map_aes32;
+        dest_map = dest_map_aes32;
+        meter_map_input = channel_map_aes32;
+        meter_map_playback = channel_map_aes32;
+
+    }
+
+    if (HDSPeRayDAT == type) {
+        playbacks_offset = 64;
+
+        switch (speed_mode) {
+        case 0: // SS
+            channels_input = 36;
+            channels_playback = 36;
+            channels_output = 36; /* SS 4x8xADAT+2xAES/EBU+2xSPDIF */
+            channel_map_input = channel_map_playback = channel_map_raydat_ss;
+            dest_map = dest_map_raydat_ss;
+            meter_map_input = meter_map_playback = channel_map_raydat_ss;
+            break;
+        case 1: // DS
+            channels_input = 20;
+            channels_playback = 20;
+            channels_output = 20; /* DS 4x4xADAT(SMUX)+2xAES/EBU+2xSPDIF */
+            channel_map_input = channel_map_playback = channel_map_raydat_ds;
+            dest_map = dest_map_raydat_ds;
+            meter_map_input = meter_map_playback = channel_map_raydat_ds;
+            break;
+        case 2: // QS
+            channels_input = 12;
+            channels_playback = 12;
+            channels_output = 12; /* QS 4x2xADAT(SMUX)+2xAES/EBU+2xSPDIF */
+            channel_map_input = channel_map_playback = channel_map_raydat_qs;
+            dest_map = dest_map_raydat_qs;
+            meter_map_input = meter_map_playback = channel_map_raydat_qs;
+            break;
+        }
 
     }
 
     window_width = (channels_playback+2)*STRIP_WIDTH;
     window_height = FULLSTRIP_HEIGHT*2+SMALLSTRIP_HEIGHT+MENU_HEIGHT;
-} 
+}
 
 void HDSPMixerCard::setMode(int mode)
 {
@@ -510,7 +443,9 @@ void HDSPMixerCard::setMode(int mode)
     }
     for (int i = 0; i < channels_playback; ++i) {
       basew->playbacks->strips[i]->targets->setLabels();
-      basew->outputs->strips[i]->setLabels();    
+    }
+    for (int i = 0; i < channels_output; ++i) {
+      basew->outputs->strips[i]->setLabels();
     }
 
     if (h9632_aeb.aebo && !h9632_aeb.aebi) {
@@ -548,11 +483,16 @@ void HDSPMixerCard::actualizeStrips()
 	} else {
 	    basew->inputs->strips[i]->hide();
 	}
+
 	if (i < channels_playback) {
 	    basew->playbacks->strips[i]->show();
-	    basew->outputs->strips[i]->show();
 	} else {
 	    basew->playbacks->strips[i]->hide();
+	}
+
+	if (i < channels_output) {
+	    basew->outputs->strips[i]->show();
+	} else {
 	    basew->outputs->strips[i]->hide();
 	}
     }
