@@ -3,6 +3,9 @@
  *    
  *   Copyright (C) 2003 Thomas Charbonnel (thomas@undata.org)
  *    
+ *   Copyright (C) 2011 Adrian Knoth (adi@drcomp.erfurt.thur.de)
+ *                      Fredrik Lingvall (fredrik.lingvall@gmail.com)
+ *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -21,7 +24,7 @@
 #pragma implementation
 #include "HDSPMixerSelector.h"
 
-static char *destinations_madi_ss[32] = {
+static char const *destinations_madi_ss[32] = {
   "1+2", "3+4", "5+6", "7+8",
   "9+10", "11+12", "13+14", "15+16",
   "17+18", "19+20", "21+22", "23+24",
@@ -32,20 +35,24 @@ static char *destinations_madi_ss[32] = {
   "57+58", "59+60", "61+62", "63+64"
 };
 
-static char *destinations_madi_ds[16] = {
+static char const *destinations_madi_ds[16] = {
   "1+2", "3+4", "5+6", "7+8",
   "9+10", "11+12", "13+14", "15+16",
   "17+18", "19+20", "21+22", "23+24",
-  "25+26", "27+28", "29+30", "31+32",
+  "25+26", "27+28", "29+30", "31+32"
 };
 
-static char *destinations_madi_qs[8] = {
+static char const *destinations_madi_qs[8] = {
   "1+2", "3+4", "5+6", "7+8",
-  "9+10", "11+12", "13+14", "15+16",
+  "9+10", "11+12", "13+14", "15+16"
 };
 
+static char const *destinations_aes32[8] = {
+  "AES 1+2", "AES 3+4", "AES 5+6", "AES 7+8",
+  "AES 9+10", "AES 11+12", "AES 13+14", "AES 15+16",
+};
 
-static char *destinations_raydat_ss[18] = {
+static char const *destinations_raydat_ss[18] = {
 
   "A1 1+2", "A1 3+4", "A1 5+6", "A1 7+8",
   "A2 1+2", "A2 3+4", "A2 5+6", "A2 7+8",
@@ -55,7 +62,7 @@ static char *destinations_raydat_ss[18] = {
   "SPDIF"
 };
 
-static char *destinations_raydat_ds[10] = {
+static char const *destinations_raydat_ds[10] = {
   "A1 1+2", "A1 3+4",
   "A2 1+2", "A2 3+4",
   "A3 1+2", "A3 3+4",
@@ -64,7 +71,7 @@ static char *destinations_raydat_ds[10] = {
   "SPDIF"
 };
 
-static char *destinations_raydat_qs[6] = {
+static char const *destinations_raydat_qs[6] = {
   "A1 1+2",
   "A2 1+2",
   "A3 1+2",
@@ -74,86 +81,103 @@ static char *destinations_raydat_qs[6] = {
 };
 
 
-static char *destinations_aio_ss[8] = {
+static char const *destinations_aio_ss[10] = {
   "AN 1+2",
   "AES",
-  "SPDIF", 
+  "SPDIF",
   "A 1+2", "A 3+4", "A 5+6", "A 7+8",
   "Phones",
+  "AEB 1+2",
+  "AEB 3+4"
 };
 
-static char *destinations_aio_ds[6] = {
+static char const *destinations_aio_ds[8] = {
   "AN 1+2", 
   "AES",
-  "SPDIF", 
-  "A 1+2", "A 3+4"
+  "SPDIF",
+  "A 1+2", "A 3+4",
   "Phones",
+  "AEB 1+2",
+  "AEB 3+4"
 };
 
-static char *destinations_aio_qs[5] = {
+static char const *destinations_aio_qs[7] = {
   "AN 1+2",
   "AES",
-  "SPDIF", 
-  "A 1+2"
+  "SPDIF",
+  "A 1+2",
   "Phones",
+  "AEB 1+2",
+  "AEB 3+4"
 };
 
-    static char *destinations_mf_ss[10] = {
-	"AN 1+2", "AN 3+4", "AN 5+6", "AN 7+8",
-	"A 1+2", "A 3+4", "A 5+6", "A 7+8",
-	"SPDIF", "Analog"
-    };
-    static char *destinations_mf_ds[8] = {
-	"AN 1+2", "AN 3+4", "AN 5+6", "AN 7+8",
-	"A 1+2", "A 3+4",
-        "SPDIF", "Analog"
-    };
-    static char *destinations_df_ss[14] = {
-	"A1 1+2", "A1 3+4", "A1 5+6", "A1 7+8",
-	"A2 1+2", "A2 3+4", "A2 5+6", "A2 7+8",
-	"A3 1+2", "A3 3+4", "A3 5+6", "A3 7+8",
-	"SPDIF", "Analog"
-    };
-    static char *destinations_df_ds[8] = {
-	"A1 1+2", "A1 3+4",
-	"A2 1+2", "A2 3+4",
-	"A3 1+2", "A3 3+4",
-	"SPDIF", "Analog"
-    };
-    static char *destinations_h9652_ss[13] = {
-	"A1 1+2", "A1 3+4", "A1 5+6", "A1 7+8",
-	"A2 1+2", "A2 3+4", "A2 5+6", "A2 7+8",
-	"A3 1+2", "A3 3+4", "A3 5+6", "A3 7+8",
-	"SPDIF"
-    };
+static char const *destinations_mf_ss[10] = {
+  "AN 1+2", "AN 3+4", "AN 5+6", "AN 7+8",
+  "A 1+2", "A 3+4", "A 5+6", "A 7+8",
+  "SPDIF", "Analog"
+};
 
-    static char *destinations_h9652_ds[7] = {
-	"A1 1+2", "A1 3+4",
-	"A2 1+2", "A2 3+4",
-	"A3 1+2", "A3 3+4",
-	"SPDIF"
-    };
-    static char *destinations_h9632_ss[8] = {
-	"A 1+2", "A 3+4", "A 5+6", "A 7+8",
-	"SPDIF", "AN 1+2", "AN 3+4", "AN 5+6"
-    };
-    static char *destinations_h9632_ds[6] = {
-	"A 1+2", "A 3+4",
-	"SPDIF", "AN 1+2", "AN 3+4", "AN 5+6"    
-    };
-    static char *destinations_h9632_qs[4] = {
-	"SPDIF", "AN 1+2", "AN 3+4", "AN 5+6"    
-    };
+static char const *destinations_mf_ds[8] = {
+  "AN 1+2", "AN 3+4", "AN 5+6", "AN 7+8",
+  "A 1+2", "A 3+4",
+  "SPDIF", "Analog"
+};
+
+static char const *destinations_df_ss[14] = {
+  "A1 1+2", "A1 3+4", "A1 5+6", "A1 7+8",
+  "A2 1+2", "A2 3+4", "A2 5+6", "A2 7+8",
+  "A3 1+2", "A3 3+4", "A3 5+6", "A3 7+8",
+  "SPDIF", "Analog"
+};
+
+static char const *destinations_df_ds[8] = {
+  "A1 1+2", "A1 3+4",
+  "A2 1+2", "A2 3+4",
+  "A3 1+2", "A3 3+4",
+  "SPDIF", "Analog"
+};
+
+static char const *destinations_rpm[3] = {
+  "Main", "Mon", "Phones"
+};
+
+static char const *destinations_h9652_ss[13] = {
+  "A1 1+2", "A1 3+4", "A1 5+6", "A1 7+8",
+  "A2 1+2", "A2 3+4", "A2 5+6", "A2 7+8",
+  "A3 1+2", "A3 3+4", "A3 5+6", "A3 7+8",
+  "SPDIF"
+};
+
+static char const *destinations_h9652_ds[7] = {
+  "A1 1+2", "A1 3+4",
+  "A2 1+2", "A2 3+4",
+  "A3 1+2", "A3 3+4",
+  "SPDIF"
+};
+
+static char const *destinations_h9632_ss[8] = {
+  "A 1+2", "A 3+4", "A 5+6", "A 7+8",
+  "SPDIF", "AN 1+2", "AN 3+4", "AN 5+6"
+};
+
+static char const *destinations_h9632_ds[6] = {
+  "A 1+2", "A 3+4",
+  "SPDIF", "AN 1+2", "AN 3+4", "AN 5+6"    
+};
+
+static char const *destinations_h9632_qs[4] = {
+  "SPDIF", "AN 1+2", "AN 3+4", "AN 5+6"    
+};
 
 HDSPMixerSelector::HDSPMixerSelector(int x, int y, int w, int h):Fl_Menu_(x, y, w, h)
 {
     max_dest = 0;
     selected = 0;
     basew = (HDSPMixerWindow *)window();
-    setLabels();
     textfont(FL_HELVETICA);
     textsize(8);
-    textcolor(FL_BLACK);
+    textcolor(FL_FOREGROUND_COLOR);
+    setLabels();
 }
 
 void HDSPMixerSelector::draw() {
@@ -247,6 +271,9 @@ void HDSPMixerSelector::setLabels()
 	    /* should never happen */
 	    break;
 	}
+    } else if (type == RPM) {
+        max_dest = 3;
+        destinations = destinations_rpm;
     } else if (type == H9652) {
 	switch (sm) {
 	case 0:
@@ -291,18 +318,21 @@ void HDSPMixerSelector::setLabels()
 	  destinations = destinations_madi_qs;
 	  break;
 	}
+    } else if (HDSP_AES == type) {
+      max_dest = 8;
+      destinations = destinations_aes32;
     } else if (HDSPeAIO == type) {
 	switch (sm) {
 	case 0:
-	  max_dest = 8;
+	  max_dest = 10;
 	  destinations = destinations_aio_ss;
 	  break;
 	case 1:
-	  max_dest = 6;
+	  max_dest = 8;
 	  destinations = destinations_aio_ds;
 	  break;
 	case 2:
-	  max_dest = 5;
+	  max_dest = 7;
 	  destinations = destinations_aio_qs;
 	  break;
 	}
