@@ -25,12 +25,18 @@
 #pragma implementation
 #include "HDSPMixerWindow.h"
     
+/* header used in .mix file */
+const char header[] = "HDSPMixer v1";
+
 static void readregisters_cb(void *arg)
 {
     int err;
     snd_hwdep_t *hw;
     hdsp_peak_rms_t hdsp_peak_rms;
     struct hdspm_peak_rms hdspm_peak_rms;
+    bool isMADI = false;
+    uint32_t *input_peaks, *playback_peaks, *output_peaks;
+    uint64_t *input_rms, *playback_rms, *output_rms;
     
     HDSPMixerWindow *w = (HDSPMixerWindow *)arg;
 
@@ -46,7 +52,9 @@ static void readregisters_cb(void *arg)
 
     if ((HDSPeMADI == w->cards[w->current_card]->type) ||
 	(HDSPeAIO == w->cards[w->current_card]->type) ||
+	(HDSP_AES == w->cards[w->current_card]->type) ||
 	(HDSPeRayDAT == w->cards[w->current_card]->type)) {
+      isMADI = true;
       if ((err = snd_hwdep_ioctl(hw, SNDRV_HDSPM_IOCTL_GET_PEAK_RMS, (void *)&hdspm_peak_rms)) < 0) {
 	fprintf(stderr, "HwDep ioctl failed. Metering stopped\n");
 	snd_hwdep_close(hw);
@@ -61,33 +69,54 @@ static void readregisters_cb(void *arg)
     }
     snd_hwdep_close(hw);
 
-    // check for speed change
-    if (hdspm_peak_rms.speed != w->cards[w->current_card]->speed_mode) {
-      w->cards[w->current_card]->setMode(hdspm_peak_rms.speed);
+    if (isMADI) {
+        // check for speed change
+	    if (hdspm_peak_rms.speed != w->cards[w->current_card]->speed_mode) {
+		    w->cards[w->current_card]->setMode(hdspm_peak_rms.speed);
+	    }
+        input_peaks = hdspm_peak_rms.input_peaks;
+        playback_peaks = hdspm_peak_rms.playback_peaks;
+        output_peaks = hdspm_peak_rms.output_peaks;
+
+        input_rms = hdspm_peak_rms.input_rms;
+        playback_rms = hdspm_peak_rms.playback_rms;
+        output_rms = hdspm_peak_rms.output_rms;
+    } else {
+        /* speed changes on non-MADI are already handled via alsactl_cb and
+         * getSpeed(), but the metering structs differ.
+         */
+        input_peaks = hdsp_peak_rms.input_peaks;
+        playback_peaks = hdsp_peak_rms.playback_peaks;
+        output_peaks = hdsp_peak_rms.output_peaks;
+
+        input_rms = hdsp_peak_rms.input_rms;
+        playback_rms = hdsp_peak_rms.playback_rms;
+        output_rms = hdsp_peak_rms.output_rms;
     }
 
+    /* update the meter */
     if (w->inputs->buttons->input) {
-      for (int i = 0; i < w->cards[w->current_card]->channels_input; ++i) {
-	w->inputs->strips[i]->meter->update(hdspm_peak_rms.input_peaks[(w->cards[w->current_card]->meter_map_input[i])] & 0xffffff00,
-					    hdspm_peak_rms.input_peaks[(w->cards[w->current_card]->meter_map_input[i])] & 0xf,
-					    hdspm_peak_rms.input_rms[(w->cards[w->current_card]->meter_map_input[i])]);
-      }
+        for (int i = 0; i < w->cards[w->current_card]->channels_input; ++i) {
+            w->inputs->strips[i]->meter->update(input_peaks[(w->cards[w->current_card]->meter_map_input[i])] & 0xffffff00,
+                    input_peaks[(w->cards[w->current_card]->meter_map_input[i])] & 0xf,
+                    input_rms[(w->cards[w->current_card]->meter_map_input[i])]);
+        }
     }
-     
+
     if (w->inputs->buttons->playback) {
-      for (int i = 0; i < w->cards[w->current_card]->channels_playback; ++i) {
-	w->playbacks->strips[i]->meter->update(hdspm_peak_rms.playback_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xffffff00,
-					       hdspm_peak_rms.playback_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xf,
-					       hdspm_peak_rms.playback_rms[(w->cards[w->current_card]->meter_map_playback[i])]);
-      }
+        for (int i = 0; i < w->cards[w->current_card]->channels_playback; ++i) {
+            w->playbacks->strips[i]->meter->update(playback_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xffffff00,
+                    playback_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xf,
+                    playback_rms[(w->cards[w->current_card]->meter_map_playback[i])]);
+        }
     }
 
     if (w->inputs->buttons->output) {
-      for (int i = 0; i < w->cards[w->current_card]->channels_playback; ++i) {
-	w->outputs->strips[i]->meter->update(hdspm_peak_rms.output_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xffffff00,
-					     hdspm_peak_rms.output_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xf,
-					     hdspm_peak_rms.output_rms[(w->cards[w->current_card]->meter_map_playback[i])]);
-      }
+        for (int i = 0; i < w->cards[w->current_card]->channels_output; ++i) {
+            w->outputs->strips[i]->meter->update(output_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xffffff00,
+                    output_peaks[(w->cards[w->current_card]->meter_map_playback[i])] & 0xf,
+                    output_rms[(w->cards[w->current_card]->meter_map_playback[i])]);
+        }
     }
 
     Fl::add_timeout(0.03, readregisters_cb, w);
@@ -190,8 +219,7 @@ static void save_cb(Fl_Widget *widget, void *arg)
 	if (!(w->file_name = fl_file_chooser("Choose a file to save presets to :", "HDSPMixer preset file (*.mix)", NULL, 0))) return;
     }
     w->save();
-    snprintf(w->window_title, FL_PATH_MAX, "HDSPMixer - %s",  fl_filename_name(w->file_name));
-    w->label(w->window_title);
+    w->setTitleWithFilename();
 }
 
 static void make_default_cb(Fl_Widget *widget, void *arg)
@@ -215,10 +243,9 @@ static void restore_defaults_cb(Fl_Widget *widget, void *arg)
     w->prefs->deleteEntry("default_file");
     w->prefs->flush();
     w->file_name = NULL;
-    snprintf(w->window_title, FL_PATH_MAX, "HDSPMixer");
-    w->label(w->window_title);
+    w->setTitleWithFilename();
     w->resetMixer();
-    while (w->cards[i] != NULL) {
+    while (i < MAX_CARDS && w->cards[i] != NULL) {
 	w->restoreDefaults(i++);
     }
     w->inputs->buttons->presets->preset_change(1);
@@ -310,29 +337,54 @@ static int handler_cb(int event)
 
 void HDSPMixerWindow::save() 
 {
+    const int pan_array_size =
+        sizeof(inputs->strips[0]->data[0][0][0]->pan_pos) /
+        sizeof(inputs->strips[0]->data[0][0][0]->pan_pos[0]);
+    /* MixerStripData defines pan_pos[HDSP_MAX_DEST], but just in case this
+     * will ever change, let's detect it early and fail safely instead of
+     * reading/writing garbage from/to preset files
+     */
+    assert (HDSP_MAX_DEST == pan_array_size);
+
+    /* also make sure that fader_pos[] has the same size as pan_pos. This comes
+     * naturally, but just to be sure.
+     */
+    assert (pan_array_size ==
+            sizeof(inputs->strips[0]->data[0][0][0]->fader_pos) /
+            sizeof(inputs->strips[0]->data[0][0][0]->fader_pos[0]));
+
+
     FILE *file;
+
     if ((file = fopen(file_name, "w")) == NULL) {
 	fl_alert("Error opening file %s for saving", file_name);
     }
     if (dirty) {
 	inputs->buttons->presets->save_preset(current_preset+1);
     }
+    /* since hdspmixer 1.11, we also store the meter level settings. Indicate
+     * the new on-disk structure via a small header */
+    if (fwrite((void *)&header, sizeof(char), sizeof(header), file) !=
+            sizeof(header)) {
+        goto save_error;
+    }
+
     for (int speed = 0; speed < 3; ++speed) {
-	for (int card = 0; card < 3; ++card) {
+	for (int card = 0; card < MAX_CARDS; ++card) {
 	    for (int preset = 0; preset < 8; ++preset) {
 		for (int channel = 0; channel < HDSP_MAX_CHANNELS; ++channel) {
 		    /* inputs pans and volumes */
-		    if (fwrite((void *)&(inputs->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fwrite((void *)&(inputs->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto save_error;
 		    }
-		    if (fwrite((void *)&(inputs->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fwrite((void *)&(inputs->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto save_error;
 		    }
 		    /* playbacks pans and volumes */
-		    if (fwrite((void *)&(playbacks->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fwrite((void *)&(playbacks->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto save_error;
 		    }
-		    if (fwrite((void *)&(playbacks->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fwrite((void *)&(playbacks->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto save_error;
 		    }
 		    /* inputs mute/solo/dest */
@@ -390,6 +442,24 @@ void HDSPMixerWindow::save()
 		if (fwrite((void *)&(data[card][speed][preset]->mute), sizeof(int), 1, file) != 1) {
 		    goto save_error;
 		}		
+		if (fwrite((void *)&(data[card][speed][preset]->last_destination), sizeof(int), 1, file) != 1) {
+		    goto save_error;
+		}
+		if (fwrite((void *)&(data[card][speed][preset]->rmsplus3), sizeof(int), 1, file) != 1) {
+		    goto save_error;
+		}
+		if (fwrite((void *)&(data[card][speed][preset]->numbers), sizeof(int), 1, file) != 1) {
+		    goto save_error;
+		}
+		if (fwrite((void *)&(data[card][speed][preset]->over), sizeof(int), 1, file) != 1) {
+		    goto save_error;
+		}
+		if (fwrite((void *)&(data[card][speed][preset]->level), sizeof(int), 1, file) != 1) {
+		    goto save_error;
+		}
+		if (fwrite((void *)&(data[card][speed][preset]->rate), sizeof(int), 1, file) != 1) {
+		    goto save_error;
+		}
 	    }
 	}
     }
@@ -407,28 +477,65 @@ void HDSPMixerWindow::load()
     if ((file = fopen(file_name, "r")) == NULL) {
 	int i = 0;
 	fl_alert("Error opening file %s for reading", file_name);
-	while (cards[i] != NULL) {
+	while (i < MAX_CARDS && cards[i] != NULL) {
 	    restoreDefaults(i++);
 	}
 	inputs->buttons->presets->preset_change(1);	
 	return;
     }
+
+    /* check for new ondisk format */
+    char buffer[sizeof(header)];
+    bool ondisk_v1 = false;
+    int pan_array_size = 14; /* old (pre 1.0.24) HDSP_MAX_DEST */
+    int channels_per_card = 26; /* old (pre 1.0.24) HDSP_MAX_CHANNELS */
+
+    if (fread(&buffer, sizeof(char), sizeof(buffer), file) != sizeof(buffer)) {
+            goto load_error;
+    }
+    if (0 == strncmp(buffer, header, sizeof(buffer))) {
+        /* new ondisk format found */
+        ondisk_v1 = true;
+        pan_array_size = HDSP_MAX_DEST;
+        channels_per_card = HDSP_MAX_CHANNELS;
+    } else {
+        /* There are two different kinds of old format: pre 1.0.24 and
+         * the one used for 1.0.24/1.0.24.1. We can distinguish between
+         * these two by checking the file size, becase HDSP_MAX_CHANNELS
+         * was bumped right before the 1.0.24 release.
+         */
+        fseek (file, 0, SEEK_END);
+        long filesize = ftell (file);
+        
+        if (1163808 == filesize) {
+            /* file written by hdspmixer v1.0.24 or v1.0.24.1 with
+             * HDSP_MAX_CHANNELS set to 64, but pan_array_size still at
+             * 14, so setting channels_per_card should get the correct
+             * mapping.
+             */
+            channels_per_card = 64; /* HDSP_MAX_CHANNELS */
+        }
+
+        /* rewind to the start and simply read all data */
+        rewind(file);
+    }
+
     for (int speed = 0; speed < 3; ++speed) {
-	for (int card = 0; card < 3; ++card) {
+	for (int card = 0; card < MAX_CARDS; ++card) {
 	    for (int preset = 0; preset < 8; ++preset) {
-		for (int channel = 0; channel < HDSP_MAX_CHANNELS; ++channel) {
+		for (int channel = 0; channel < channels_per_card; ++channel) {
 		    /* inputs pans and volumes */
-		    if (fread((void *)&(inputs->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fread((void *)&(inputs->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto load_error;
 		    }
-		    if (fread((void *)&(inputs->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fread((void *)&(inputs->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto load_error;
 		    }
 		    /* playbacks pans and volumes */
-		    if (fread((void *)&(playbacks->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fread((void *)&(playbacks->strips[channel]->data[card][speed][preset]->pan_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto load_error;
 		    }
-		    if (fread((void *)&(playbacks->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), 14, file) != 14) {
+		    if (fread((void *)&(playbacks->strips[channel]->data[card][speed][preset]->fader_pos[0]), sizeof(int), pan_array_size, file) != pan_array_size) {
 			goto load_error;
 		    }
 		    /* inputs mute/solo/dest */
@@ -486,12 +593,32 @@ void HDSPMixerWindow::load()
 		if (fread((void *)&(data[card][speed][preset]->mute), sizeof(int), 1, file) != 1) {
 		    goto load_error;
 		}		
+        /* read additional meter settings only present in newer mix files */
+        if (ondisk_v1) {
+            if (fread((void *)&(data[card][speed][preset]->last_destination), sizeof(int), 1, file) != 1) {
+                goto load_error;
+            }
+            if (fread((void *)&(data[card][speed][preset]->rmsplus3), sizeof(int), 1, file) != 1) {
+                goto load_error;
+            }
+            if (fread((void *)&(data[card][speed][preset]->numbers), sizeof(int), 1, file) != 1) {
+                goto load_error;
+            }
+            if (fread((void *)&(data[card][speed][preset]->over), sizeof(int), 1, file) != 1) {
+                goto load_error;
+            }
+            if (fread((void *)&(data[card][speed][preset]->level), sizeof(int), 1, file) != 1) {
+                goto load_error;
+            }
+            if (fread((void *)&(data[card][speed][preset]->rate), sizeof(int), 1, file) != 1) {
+                goto load_error;
+            }
+        }
 	    }
 	}
     }
     fclose(file);
-    snprintf(window_title, FL_PATH_MAX, "HDSPMixer - %s",  fl_filename_name(file_name));
-    label(window_title);
+    setTitleWithFilename();
     resetMixer();
     inputs->buttons->presets->preset_change(1);
     return;
@@ -501,6 +628,66 @@ load_error:
     return;
 }
 
+void HDSPMixerWindow::setTitle(std::string suffix)
+{
+    std::string title = "HDSPMixer (";
+
+    title = title + cards[current_card]->cardname + ") "; /*cardname */
+    title = title + suffix;
+    snprintf(window_title, FL_PATH_MAX, "%s", title.c_str());
+    label(window_title);
+}
+
+void HDSPMixerWindow::setTitleWithFilename(void)
+{
+    const char *filename = fl_filename_name(file_name);
+
+    if (NULL == file_name) {
+        filename = "(unsaved)";
+    }
+
+    setTitle(filename);
+}
+
+void HDSPMixerWindow::stashPreset(void)
+{
+    cards[current_card]->last_preset = current_preset;
+    cards[current_card]->last_dirty = dirty;
+    /* save the current mixer state to the virtual 9th preset */
+    inputs->buttons->presets->save_preset(9);
+}
+
+void HDSPMixerWindow::unstashPreset(void)
+{
+    /* load temporary data from virtual 9th preset */
+    inputs->buttons->presets->restore_preset(9);
+    
+    current_preset = cards[current_card]->last_preset;
+    /* Internal notion of playback in use. Relevant for blinking buttons */
+    inputs->buttons->presets->preset = current_preset + 1;
+    dirty = cards[current_card]->last_dirty;
+    /* Preset masks (which preset button is green) */
+    inputs->buttons->presets->presetmask = (int)pow(2, current_preset);
+    if (dirty) {
+        /* make the buttons blink if it's unsaved. We need to remove any
+         * existing triggers to dirty_cb, because dirty_cb is called
+         * every 0.3 seconds and enabling/disabling (highlight/unlight) the
+         * buttons, so if we have too many callbacks, the buttons would
+         * remain in one state --> no blinking. We need exactly one.
+         */
+        Fl::remove_timeout(dirty_cb, (void *)this);
+        Fl::add_timeout(0.3, dirty_cb, (void *)this);
+    } else {
+        /* Hack. I don't know why this is necessary, but if we're clean,
+         * we need to recall the preset again to correctly reflect
+         * the dirty/undirty state.
+         *
+         * Though it's a little bit redundant, it at least won't do any harm.
+         */
+        inputs->buttons->presets->preset_change(current_preset+1);
+    }
+}
+
 void HDSPMixerWindow::restoreDefaults(int card)
 {
     int chnls[3];
@@ -522,6 +709,10 @@ void HDSPMixerWindow::restoreDefaults(int card)
 	maxdest[0] = 14;
 	maxdest[1] = 8;
 	break;
+    case RPM:
+    chnls[0] = chnls[1] = 6;
+    maxdest[0] = maxdest[1] = 3;
+    break;
     case H9652:
 	chnls[0] = 26;
 	chnls[1] = 14;
@@ -552,13 +743,22 @@ void HDSPMixerWindow::restoreDefaults(int card)
       maxdest[2] = 8;
       num_modes = 3;
       break;
-     case HDSPeAIO:
-      chnls[0] = 14;
-      chnls[1] = 10;
-      chnls[2] = 8;
+    case HDSP_AES: /* these cards support full channel count at all modes */
+      chnls[0] = 16;
+      chnls[1] = 16;
+      chnls[2] = 16;
       maxdest[0] = 8;
-      maxdest[1] = 6;
-      maxdest[2] = 5;
+      maxdest[1] = 8;
+      maxdest[2] = 8;
+      num_modes = 3;
+      break;
+     case HDSPeAIO:
+      chnls[0] = 18;
+      chnls[1] = 14;
+      chnls[2] = 12;
+      maxdest[0] = 10;
+      maxdest[1] = 8;
+      maxdest[2] = 7;
       num_modes = 3;
       break;
     case HDSPeRayDAT:
@@ -575,9 +775,10 @@ void HDSPMixerWindow::restoreDefaults(int card)
 	/* should never happen */
 	return;
     }
+
     for (int preset = 0; preset < 8; ++preset) {
 	for (int speed = 0; speed < num_modes; ++speed) {
-	    for (int i = 0; i < chnls[speed]; i+=2) {
+	    for (int i = 0; i < 2*maxdest[speed]; i+=2) {
     		for (int z = 0; z < maxdest[speed]; ++z) {
 		    /* Gain setup */
 		    if (cards[card]->type == H9632) {
@@ -670,8 +871,8 @@ HDSPMixerWindow::HDSPMixerWindow(int x, int y, int w, int h, const char *label,
 	    prefs->flush();
 	}	
     }
-    for (int j = 0; j < 3; ++j) {
-	for (int i = 0; i < 8; ++i) {
+    for (int j = 0; j < MAX_CARDS; j++) {
+	for (int i = 0; i < NUM_PRESETS; ++i) {
 	    data[j][0][i] = new HDSPMixerPresetData();
 	    data[j][1][i] = new HDSPMixerPresetData();
 	    data[j][2][i] = new HDSPMixerPresetData();
@@ -702,13 +903,13 @@ HDSPMixerWindow::HDSPMixerWindow(int x, int y, int w, int h, const char *label,
     inputs->buttons->submix = 1;
     inputs->buttons->playback = 1;
     playbacks = new HDSPMixerPlaybacks(0, MENU_HEIGHT+FULLSTRIP_HEIGHT, w, FULLSTRIP_HEIGHT, cards[0]->channels_playback);
-    outputs = new HDSPMixerOutputs(0, MENU_HEIGHT+FULLSTRIP_HEIGHT*2, w, SMALLSTRIP_HEIGHT, cards[0]->channels_playback);
+    outputs = new HDSPMixerOutputs(0, MENU_HEIGHT+FULLSTRIP_HEIGHT*2, w, SMALLSTRIP_HEIGHT, cards[0]->channels_output);
     scroll->end();
     end();
     setup = new HDSPMixerSetup(400, 260, "Level Meters Setup", this);
-    about = new HDSPMixerAbout(360, 260, "About HDSPMixer", this);
+    about = new HDSPMixerAbout(360, 300, "About HDSPMixer", this);
     i = 0;
-    while (cards[i] != NULL) {
+    while (i < MAX_CARDS && cards[i] != NULL) {
 	cards[i++]->initializeCard(this);
     }
     size_range(MIN_WIDTH, MIN_HEIGHT, cards[current_card]->window_width, cards[current_card]->window_height);
@@ -719,17 +920,20 @@ HDSPMixerWindow::HDSPMixerWindow(int x, int y, int w, int h, const char *label,
     } else {
 	printf("Initializing default presets\n");
 	i = 0;
-	while (cards[i] != NULL) {
+	while (i < MAX_CARDS && cards[i] != NULL) {
+	    current_card = i;
 	    restoreDefaults(i++);
+	    inputs->buttons->presets->preset_change(1);
 	}
-	inputs->buttons->presets->preset_change(1);
     }
     Fl::atclose = atclose_cb;
     Fl::add_handler(handler_cb);
     Fl::add_timeout(0.030, readregisters_cb, this);
     i = 0;
-    while (cards[i] != NULL)
+    while (i < MAX_CARDS && cards[i] != NULL) {
+      current_card = i;
       inputs->buttons->cardselector->ActivateCard (i++);
+    }
 }
 
 int HDSPMixerWindow::handle(int e) 
@@ -847,16 +1051,14 @@ void HDSPMixerWindow::checkState()
 	corrupt++;
 
     if (corrupt) {
-	if (!dirty) {
-	    dirty = 1;
-	    snprintf(window_title, FL_PATH_MAX, "HDSPMixer - %s *",  fl_filename_name(file_name));
-	    label(window_title);
-	    Fl::add_timeout(0.3, dirty_cb, (void *)this); 
-	}
+        if (!dirty) {
+            dirty = 1;
+            setTitleWithFilename();
+            Fl::add_timeout(0.3, dirty_cb, (void *)this); 
+        }
     } else {
-	snprintf(window_title, FL_PATH_MAX, "HDSPMixer - %s",  fl_filename_name(file_name));
-	label(window_title);
-	dirty = 0;
+        setTitleWithFilename();
+        dirty = 0;
     }
 }
 
