@@ -27,6 +27,7 @@
 #define HDSPeMADI 10
 #define HDSPeRayDAT 11
 #define HDSPeAIO 12
+#define HDSP_AES 13 /* both HDSP AES32 and HDSPe AES */
 
 #define HDSP_MAX_CHANNELS 64
 #define HDSP_MAX_DEST	  32
@@ -46,6 +47,14 @@
 
 #define PAN_WIDTH 28
 
+#define MAX_CARDS	3
+
+/* Number of presets. 8 presets visible to the user, the 9th is used for
+ * holding temporary mixer data when switching cards, so it's not a real
+ * preset but more like a scratch pad.
+ */
+#define NUM_PRESETS	9
+
 typedef unsigned long long int int64;
 
 #endif
